package com.truiton.mobile.vision.ocr;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.util.SparseArray;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.vision.Frame;
import com.google.android.gms.vision.text.Text;
import com.google.android.gms.vision.text.TextBlock;
import com.google.android.gms.vision.text.TextRecognizer;

import anywheresoftware.b4a.BA;
import mobilevisiontextwrapper.mobilevisiontextWrapper;

import java.io.File;
import java.io.FileNotFoundException;

public class MainActivity extends AppCompatActivity {
    private static final String LOG_TAG = "Text API";
    private static final int PHOTO_REQUEST = 10;
    private TextView scanResults;
    private Uri imageUri;
    private TextRecognizer detector;
    private static final int REQUEST_WRITE_PERMISSION = 20;
    private static final String SAVED_INSTANCE_URI = "uri";
    private static final String SAVED_INSTANCE_RESULT = "result";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Button button = (Button) findViewById(R.id.button);
        scanResults = (TextView) findViewById(R.id.results);
        if (savedInstanceState != null) {
            imageUri = Uri.parse(savedInstanceState.getString(SAVED_INSTANCE_URI));
            scanResults.setText(savedInstanceState.getString(SAVED_INSTANCE_RESULT));
        }
        detector = new TextRecognizer.Builder(getApplicationContext()).build();
		takePicture();
        //button.setOnClickListener(new View.OnClickListener() {
        //    @Override
        //    public void onClick(View view) {
        //        ActivityCompat.requestPermissions(MainActivity.this, new
        //                String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, REQUEST_WRITE_PERMISSION);
        //    }
        //});
    }

/*     @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_WRITE_PERMISSION:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    takePicture();
                } else {
                    Toast.makeText(MainActivity.this, "Permission Denied!", Toast.LENGTH_SHORT).show();
                }
        }
    } */

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		//BA.Log("in onActivityResult");
        if (requestCode == PHOTO_REQUEST && resultCode == RESULT_OK) {
            launchMediaScanIntent();
            try {
                Bitmap bitmap = decodeBitmapUri(this, imageUri);
				//BA.Log("bitmap = " + bitmap);
                if (detector.isOperational() && bitmap != null) {
					//BA.Log("detector is operational");
                    Frame frame = new Frame.Builder().setBitmap(bitmap).build();
                    SparseArray<TextBlock> textBlocks = detector.detect(frame);
                    String blocks = "";
                    String lines = "";
                    String words = "";
                    for (int index = 0; index < textBlocks.size(); index++) {
                        //extract scanned text blocks here
                        TextBlock tBlock = textBlocks.valueAt(index);
                        blocks = blocks + tBlock.getValue() + "\n" + "\n";
                        for (Text line : tBlock.getComponents()) {
                            //extract scanned text lines here
                            lines = lines + line.getValue() + "\n";
                            for (Text element : line.getComponents()) {
                                //extract scanned text words here
                                words = words + element.getValue() + ", ";
								//BA.Log("words = " + words);
                            }
                        }
                    }
                    if (textBlocks.size() == 0) {
                        scanResults.setText("Scan Failed: Found nothing to scan");
                    } else {
                        //scanResults.setText(scanResults.getText() + "Blocks: " + "\n");
                        scanResults.setText(scanResults.getText() + blocks + "\n");
						if (mobilevisiontextWrapper.ba.subExists(mobilevisiontextWrapper.eventName + "_blocks_result")) {
							mobilevisiontextWrapper.ba.raiseEventFromDifferentThread(this, null, 0, mobilevisiontextWrapper.eventName + "_blocks_result", true, new Object[] 
											{blocks});
						}							
						
						
                        //scanResults.setText(scanResults.getText() + "---------" + "\n");
                        //scanResults.setText(scanResults.getText() + "Lines: " + "\n");
                        scanResults.setText(scanResults.getText() + lines + "\n");
						if (mobilevisiontextWrapper.ba.subExists(mobilevisiontextWrapper.eventName + "_lines_result")) {
							mobilevisiontextWrapper.ba.raiseEventFromDifferentThread(this, null, 0, mobilevisiontextWrapper.eventName + "_lines_result", true, new Object[] 
											{lines});
						}													
						
                        //scanResults.setText(scanResults.getText() + "---------" + "\n");
                        //scanResults.setText(scanResults.getText() + "Words: " + "\n");
                        scanResults.setText(scanResults.getText() + words + "\n");
						if (mobilevisiontextWrapper.ba.subExists(mobilevisiontextWrapper.eventName + "_words_result")) {
							mobilevisiontextWrapper.ba.raiseEventFromDifferentThread(this, null, 0, mobilevisiontextWrapper.eventName + "_words_result", true, new Object[] 
											{words});
						}							
                        //scanResults.setText(scanResults.getText() + "---------" + "\n");
						//BA.Log("scanResults = " + scanResults.getText());
                    }
                } else {
					if (mobilevisiontextWrapper.ba.subExists(mobilevisiontextWrapper.eventName + "_error_result")) {
						mobilevisiontextWrapper.ba.raiseEventFromDifferentThread(this, null, 0, mobilevisiontextWrapper.eventName + "_error_result", true, new Object[] 
										{"Could not set up the detector!"});
					}									
					
                    //scanResults.setText("Could not set up the detector!");
                }
            } catch (Exception e) {
					if (mobilevisiontextWrapper.ba.subExists(mobilevisiontextWrapper.eventName + "_error_result")) {
						mobilevisiontextWrapper.ba.raiseEventFromDifferentThread(this, null, 0, mobilevisiontextWrapper.eventName + "_error_result", true, new Object[] 
										{"Failed to load Image" + e.toString()});
					}									
                //Log.e(LOG_TAG, e.toString());
            }
        }
		finish();
    }

    private void takePicture() {
        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        File photo = new File(Environment.getExternalStorageDirectory(), "picture.jpg");
        imageUri = Uri.fromFile(photo);
        intent.putExtra(MediaStore.EXTRA_OUTPUT, imageUri);
        startActivityForResult(intent, PHOTO_REQUEST);
    }

/*  @Override
    protected void onSaveInstanceState(Bundle outState) {
		BA.Log("in onSaveInstanceState");
        if (imageUri != null) {
            outState.putString(SAVED_INSTANCE_URI, imageUri.toString());
            outState.putString(SAVED_INSTANCE_RESULT, scanResults.getText().toString());
        }
        super.onSaveInstanceState(outState);
    } */

    private void launchMediaScanIntent() {
        Intent mediaScanIntent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
        mediaScanIntent.setData(imageUri);
        this.sendBroadcast(mediaScanIntent);
    }

    private Bitmap decodeBitmapUri(Context ctx, Uri uri) throws FileNotFoundException {
        int targetW = 600;
        int targetH = 600;
        BitmapFactory.Options bmOptions = new BitmapFactory.Options();
        bmOptions.inJustDecodeBounds = true;
        BitmapFactory.decodeStream(ctx.getContentResolver().openInputStream(uri), null, bmOptions);
        int photoW = bmOptions.outWidth;
        int photoH = bmOptions.outHeight;

        int scaleFactor = Math.min(photoW / targetW, photoH / targetH);
        bmOptions.inJustDecodeBounds = false;
        bmOptions.inSampleSize = scaleFactor;

        return BitmapFactory.decodeStream(ctx.getContentResolver()
                .openInputStream(uri), null, bmOptions);
    }
}
